#include "valve_utils.h"
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#ifdef _WIN32
#include <windows.h>
#include <Psapi.h>
#endif
#ifdef __APPLE__
#include <ctype.h>
#include <TargetConditionals.h>
#if !defined( TARGET_OS_IOS ) && !defined( TARGET_OS_TV )
#include <libproc.h>
#endif
#include <unistd.h>
#endif
#ifdef __linux__
#include <dlfcn.h>
#include <linux/auxvec.h>
typedef unsigned long (getauxval_fn)(unsigned long);
#endif

#ifdef _WIN32
int valve_load_and_check_winh264_dll(void); // from valve_h264.c
#endif

int use_valve_h264( void )
{
	static int s_bDetected = 0, s_bSupported = 0;
	if ( s_bDetected )
		return s_bSupported;

#ifdef _WIN32
	s_bSupported = ( valve_load_and_check_winh264_dll() >= 0 );
#endif

	s_bDetected = 1;
	return s_bSupported;
}

static int is_steam_process(void)
{
#ifdef _WIN32
		char exename[2048];
		exename[0] = '\0';
		if (!GetModuleFileNameA( NULL, (LPSTR)&exename, sizeof(exename) ) )
				return 0;

		// lowercase
		for ( int i = 0; exename[i] != '\0'; i++ )
                    exename[i] = tolower( exename[i] );

                if ( strstr( exename, "steam.exe" ) != NULL )
                    return 1;

                if ( strstr( exename, "steamwebhelper.exe" ) != NULL )
                    return 1;

                if ( strstr( exename, "server.exe" ) != NULL )
                    return 1;

                if ( strstr( exename, "streaming_client.exe" ) != NULL )
                    return 1;

                return 0;

#elif defined(__APPLE__) && !defined( TARGET_OS_IOS ) && !defined( TARGET_OS_TV )
		char szProcPath[PROC_PIDPATHINFO_MAXSIZE];
		if ( proc_pidpath( getpid(), szProcPath, sizeof(szProcPath) - 1) == 0 )
				return 0;

		for ( int i = 0; szProcPath[i] != '\0'; i++ )
				szProcPath[i] = tolower( szProcPath[i] );

                if ( strstr( szProcPath, "steam_osx" ) != NULL )
                    return 1;

                if ( strstr( szProcPath, "steam helper" ) != NULL )
                    return 1;

                if ( strstr( szProcPath, "streaming_client" ) != NULL )
                    return 1;

		return 0;

#elif defined(__linux__) && !defined(__ANDROID__)
		int ret = 0;
		char *exename = NULL;
		getauxval_fn *pfn = dlsym( NULL, "getauxval" );
		if ( !pfn )
				return 0;
		exename = strdup((char *)pfn(AT_EXECFN));
		if ( !exename )
				return 0;

                if ( strstr( exename, "steam" ) != NULL )
                    ret = 1;

                if ( strstr( exename, "steamwebhelper" ) != NULL )
                    ret = 1;

                if ( strstr( exename, "streaming_client" ) != NULL )
                    ret = 1;

		free(exename);
		return ret;
#else
		return 0;
#endif
}

static int s_unlocked_h264 = 0;
int h264_unlocked( void )
{
	char bFreeMem = 0;
	const char *pchUnlockCode = NULL;

	if ( use_valve_h264() )
		return 1;

    if ( s_unlocked_h264 )
        return 1;
		
		if ( !is_steam_process() )
				return 0;

#ifdef _WIN32
		// on windows, getenv operates against a at-launch-time-copy of
		// the environment block.
		pchUnlockCode = (char *)calloc( 512, 1 );
		GetEnvironmentVariableA( "STEAMVIDEOTOKEN", (LPSTR)pchUnlockCode, 512 );
		bFreeMem = 1;
#else
		pchUnlockCode = getenv( "STEAMVIDEOTOKEN" );
#endif
	
	
	if ( pchUnlockCode )
	{
		unsigned int pid = 0;
		
		// consume the PID in the token and ignore it.
		sscanf( pchUnlockCode, "%u ", &pid );
			
		if ( strstr( pchUnlockCode, "32f5h290g53047gv5034nbvt923b" ) )
			s_unlocked_h264 = 1;
		
		if ( bFreeMem )
		{
			// Save to const_cast here because we allocated above
			free( (char *)pchUnlockCode );
		}
	}
	
    return s_unlocked_h264;
}
