#pragma once

#ifdef WINH264_EXPORTS
#define WINH264_API __declspec(dllexport)
#else
#define WINH264_API __declspec(dllimport)
#endif

#ifdef __cplusplus
extern "C" {
#endif

struct IDirect3DDeviceManager9;
struct IDirect3DSurface9;

typedef struct winh264_handle_t *H264DEC;
#define WINH264_INVALID ((H264DEC)0)

// int error codes
#define WINH264_OK						0		// (everything is currently copacetic)
#define WINH264_ERR_WOULD_BLOCK			(-1)	// ProcessInput: input buffer full; AcquireOutput: no frames ready
#define WINH264_ERR_NO_MORE_OUTPUT		(-2)	// WaitForOutput, AcquireOutput: end of video stream
#define WINH264_ERR_TOO_MANY_BUFFERS	(-3)	// AcquireOutput: too many output buffers are currently unreleased
#define WINH264_ERR_INCOMPATIBLE		(-4)	// AcquireOutput: decoder was created in opposite mode (D3D9 vs software)
#define WINH264_ERR_UNKNOWN_FATAL		(-5)	// (something has gone wrong, please destroy the decoder)

// CreateDecoder and DestroyDecoder must be called from the application main thread.
WINH264_API H264DEC WinH264CreateDecoder();
//WINH264_API H264DEC WinH264CreateDecoderD3D9( IDirect3DDeviceManager9 *pIDirect3DDeviceManager9 );
WINH264_API void WinH264DestroyDecoder( H264DEC hDec );

// ProcessInput, WaitForOutput, and AcquireOutput/ReleaseOutput can be called from any thread.
WINH264_API void WinH264DecoderProcessTimestamps( H264DEC hDec, __int64 llPresentTimestamp, __int64 llDecodeTimestamp ); // applies to following input NALs
WINH264_API int WinH264DecoderProcessInput( H264DEC hDec, const void *pNALU, unsigned int cbNALU );
WINH264_API void WinH264DecoderEndOfInput( H264DEC hDec );
WINH264_API int WinH264DecoderWaitForOutput( H264DEC hDec, unsigned int unMillisec ); // pass (uint)-1 for infinite wait

// NOTE: AcquireOutput and AcquireOutputD3D9 measure timestamps in 100ns increments since the initial frame; -1 indicates unknown time
WINH264_API int WinH264DecoderAcquireOutput( H264DEC hDec, const void **ppBuffer, unsigned int *punRowStride, unsigned int *punWidth, unsigned int *punHeight, __int64 *pllTimestamp, __int64 *pllDuration );

// NOTE: AcquireOutputD3D9 adds a reference to ppDirect3DSurface9; you must Release() the surface in addition to calling ReleaseOutput
// WINH264_API int WinH264DecoderAcquireOutputD3D9( H264DEC hDec, IDirect3DSurface9 **ppDirect3DSurface9, const void **ppBufferToRelease, __int64 *pllTimestamp, __int64 *pllDuration );

WINH264_API void WinH264DecoderReleaseOutput( H264DEC hDec, const void *pBuffer );

#ifdef __cplusplus
}
#endif
